import { Scene } from '@babylonjs/core/scene';
import { ISceneComponent } from '@babylonjs/core/sceneComponent';
import { MToonMaterial } from './mtoon-material';
/**
 * MToon outline renderer
 * @see OutlineRenderer
 */
export declare class MToonOutlineRenderer implements ISceneComponent {
    readonly scene: Scene;
    readonly material: MToonMaterial;
    /**
     * Stencil value used to avoid outline being seen within the mesh when the mesh is transparent
     */
    private static _StencilReference;
    static rendererId: number;
    /**
     * @inheritdoc
     */
    readonly name: string;
    /**
     * Defines a zOffset default Factor to prevent zFighting between the overlay and the mesh.
     */
    zOffset: number;
    /**
      * Defines a zOffset default Unit to prevent zFighting between the overlay and the mesh.
      */
    zOffsetUnits: number;
    private _engine;
    private _savedDepthWrite;
    private _passIdForDrawWrapper;
    /**
     * @inheritdoc
     * MToonMaterial ごとにインスタンスを生成する
     */
    constructor(scene: Scene, material: MToonMaterial);
    /**
     * @inheritdoc
     * シーン描画前後にレンダリング処理を登録する
     */
    register(): void;
    /**
     * @inheritdoc
     */
    rebuild(): void;
    /**
     * @inheritdoc
     */
    dispose(): void;
    /**
     * Renders the outline in the canvas.
     * @param subMesh Defines the sumesh to render
     * @param batch Defines the batch of meshes in case of instances
     * @param useOverlay Defines if the rendering is for the overlay or the outline
     * @param renderPassId Render pass id to use to render the mesh
     */
    private render;
    /**
     * このメッシュを描画する前に実行されるコールバック
     */
    private _beforeRenderingMesh;
    /**
     * このメッシュを描画した後に実行されるコールバック
     */
    private _afterRenderingMesh;
    /**
     * インスタンシングを行うかどうか
     */
    private isHardwareInstancedRendering;
    /**
     * このメッシュでアウトラインを描画するかどうか
     */
    private willRender;
}
